
      PROGRAM pressao

!***********************************************************************

!    Obteno da presso de combusto a partir do empuxo experimental
 
!    CARLOS HENRIQUE MARCHI - DEMEC/UFPR
!    CURITIBA, 30 Jun 05                              
!                          ltima alterao: 30 Jun 05
!***********************************************************************
 
      use portlib
 
      IMPLICIT REAL*8(A-H,O-Z)

      real*8 Me

      integer chamada
 
      chamada = system('notepad pressao.ent')
 
      OPEN(5,FILE='pressao.ent',FORM='FORMATTED')

      OPEN(6,FILE='pressao.txt',FORM='FORMATTED')

      pi = DACOS(-1.0D0)

      Pc = 100 * 1.0d+5

      CALL DADOS ( Empuxo, Pa, Ee, gama, Dt, Itmax, Tolp )

      At = pi * (Dt**2) / 4

      write(6,1)
    1 format(///,5x,'Soluo com Newton-Raphson do Mach na sada',/)

      CALL NEWTON ( gama, Ee, 1, Me, 1)

      WRITE(6,2) Me, At

    2 FORMAT(/,  1pe20.9,' = Nmero de Mach na sada (adim)', &
             /,  1pe20.9,' = rea da garganta da tubeira (m2)', &
             //, T3,'Iterao', T14,'Pe (Pa)', T34,'Cf (adim)', T54,'Pc (Pa)', T74,'Dif (Pa)')

      Fa = 1.0D0 + (gama - 1.0D0) * (Me**2) / 2.0D0

      e1 = gama / (1.0d0 - gama)

      e2 = (gama + 1.0d0) / (gama - 1.0d0)
	  
      e3 = (gama - 1.0d0) / gama

      c1 = 2 * (gama**2) / (gama - 1.0d0)

      c2 = ( 2.0d0 / ( gama + 1.0d0 ) ) ** e2
 
      DO I = 1, Itmax

         Pca = Pc

         Pe  = Pc * ( Fa ** e1 )

         Cf  = dsqrt( (c1*c2) * (1.0d0 - ((Pe/Pc)**e3) ) ) + (Pe-Pa)*Ee/Pc

         Pc  = Empuxo / ( Cf * At )

         Dif = dabs ( Pc - Pca )

         Write (6,3) I, Pe, Cf, Pc, Dif

         3 format (I8, 4(1pe20.9))

         If ( Dif < Tolp ) Exit

      End do 

      close(6)

      chamada = system ('notepad pressao.txt')
 
      contains
 
!***********************************************************************
! *** DADOS ***
!              LE OS DADOS DO PROBLEMA
!***********************************************************************

      SUBROUTINE DADOS ( Empuxo, Pa, Ee, gama, Dt, Itmax, Tolp )

      IMPLICIT REAL*8(A-H,O-Z)
 
      READ(5,*) Empuxo
      READ(5,*) Pa
      READ(5,*) Ee
      READ(5,*) gama
      READ(5,*) Dt
      READ(5,*) Itmax
      READ(5,*) Tolp
 
      WRITE(6,35)
      WRITE(6,40) Empuxo, Pa, Ee, gama, Dt, Itmax, Tolp
 
   35 FORMAT(///,5X,'DADOS',//)
   40 FORMAT(3X,1PE17.9,' = Empuxo (N)',/, &
             3X,1PE17.9,' = Presso ambiente (Pa)',/, &
             3X,1pe17.9,' = Razo de reas na sada da tubeira (adim)',/, &
             3X,1pe17.9,' = Razo entre os calores especficos (adim.)',/, &
             3X,1pe17.9,' = Dimetro da garganta da tubeira (m)',/, &
             3X,I10,7X, ' = Nmero mximo de iteraes',/, &
             3X,1pe17.9,' = Tolerncia na presso (Pa)',/)

      END SUBROUTINE

!***********************************************************************
! *** NEWTON ***
!               DETERMINA O NUMERO DE MACH EM FUNCAO DA RAZAO DE AREAS
!               USANDO O METODO DE NEWTON-RAPHSON
!***********************************************************************

      SUBROUTINE NEWTON(GAMA,A,IND,M,INP)

      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 M
 
      FDEN(M,E,C) =  M * ( 1.0D0 + E * M * M) ** C
      F(DEN,D,A) = D / DEN - A
      G(E,M) = 1.0D0 + E * M * M
 
      NITM = 500
      TOL = 1.0D-14
 
      IF(IND.EQ.1)M = 1.2D0
      IF(IND.EQ.0)M = 0.4D0

      E = (GAMA - 1.0D0) / 2.0D0
      C = (GAMA + 1.0D0) / ( 2.0D0 - 2.0D0 * GAMA)
      D = ((GAMA + 1.0D0) / 2.0D0) ** C

      DO I = 1,NITM
        FAT1 = M * ( C * G(E,M) ** (C - 1.0D0) * 2.0D0 * E * M)
        DEN = FDEN(M,E,C)
        FLINHA = - D * (G(E,M) ** C + FAT1) / (DEN * DEN)
        DELTAM = - F(DEN,D,A) / FLINHA
        M = M + DELTAM
        IF(M.LT.0.0D0) M = 0.01D0
        IF(DABS(DELTAM).LT.TOL) EXIT
      END DO

      WRITE(6,5) INP,I,DELTAM
    5 FORMAT(6X,i6,' = ponto',5X,i6,' = iteraes', &
                              5X,1pe10.2,' = variao')
 
      END SUBROUTINE

 
      END program

